/*! \file dist.cpp
 *  \brief Some functions for constructing distributions on permutations
 */

#include "dist.h"
#include "util.h"

#include <cmath>

using namespace std;

Func *uniform(int n)
{
        Func *d = new Func(n);

        for (int i = 0; i < d->funclen; i++)
                d->func[i] = 1. / d->funclen;

        return d;
}

Func *mix_subset(int n, int k, Permutation::perm_val_t *subset)
{
        Func *d = new Func(n);

        /// \todo check k <= n
        /// \todo check that subset makes sense

        int j, m;
        for (int i = 0; i < d->funclen; i++) {
                Permutation p = Permutation::from_index(n, i);

                for (j = 0; j < n; j++) {
                        for (m = 0; m < k; m++)
                                if (p(subset[m]) == j)
                                        break;
                        if (m == k)
                                if (p(j) != j)
                                        break;
                }
                if (j == n)
                        d->func[i] = 1./fact(k);
                else
                        d->func[i] = 0;
        }

        return d;
}

Func *mix_pair(int n, int i, int j, Func::val_t p)
{
        /// \todo Check i, j, p
        Func *d = new Func(n);

        int id_ind = Permutation::identity(n).index();
        int swap_ind = Permutation::swap(n, i, j).index();

        for (int k = 0; k < d->funclen; k++) {
                if (k == id_ind)
                        d->func[k] = p;
                else if (k == swap_ind)
                        d->func[k] = 1 - p;
                else
                        d->func[k] = 0;
        }

        return d;
}

Func *insertion_mix(int n)
{
        Func *d = new Func(n);

        int j;
        for (int i = 0; i < d->funclen; i++) {
                Permutation p = Permutation::from_index(n, i);
                int top;

                top = p(0);
                // check that everything after top maps to itself
                for (j = top + 1; j < n; j++)
                        if (p(j) != j)
                                break;
                if (j != n) {
                        d->func[i] = 0;
                        continue;
                }

                // check that everything before top is a cycle
                for (j = 1; j < top; j++)
                        if (p(j) != j - 1)
                                break;
                if (j < top) {
                        d->func[i] = 0;
                        continue;
                }

                d->func[i] = 1. / n;
        }

        return d;
}

Func *mallows(Permutation p0, double c)
{
        Func *d = new Func(p0.num_elements());

        for (int i = 0; i < d->funclen; i++) {
                Permutation p = Permutation::from_index(p0.num_elements(), i);

                d->func[i] =
                       exp(-c * p.inverse().of(p0).factor_adj_swaps().size());
        }

        d->normalize();

        return d;
}
