# sntools example python code / simple functionality test

from sntools import *
from numpy import dot

# create a permutation by explicit specification
print "Explicit creation:", Permutation([1, 0, 2, 3, 5, 4])

# create the identity permutation
print "Identity on 5 elements:", Permutation.identity(5)

# compose permutations
a = Permutation([0, 1, 3, 2])
b = Permutation([1, 3, 2, 0])
print "Composition:", a, "o", b, "=", a.of(b)

# find the inverse of a permutation
a = Permutation([3, 0, 2, 1])
print "Inverse:", a.__str__() + "^-1 =", a.inverse()

# direct sum of permutations
a = Permutation([0, 2, 1])
b = Permutation([4, 0, 3, 1, 2])
print "Direct sum:", a, "(+)", b, "=", a.direct_sum(b)

print "Contiguous cycle:", Permutation.cont_cycle(5, 1, 4)

# the index bijection
print "Listing permutations on 4 elements"
for i in range(fact(4)):
        p = Permutation.from_index(4, i)
        print p.index(), p

# factoring
print "Factoring into transpositions"
for p in Permutation([3, 0, 2, 1]).factor_swaps():
        print p

print "Factoring into adjacent transpositions"
for p in Permutation([3, 0, 2, 1]).factor_adj_swaps():
        print p

# partitions and tableau
print "Partition construction"
print Partition([4, 3, 2, 1, 1])

print "Partition concatenation"
p = Partition([4, 3])
q = Partition([3, 1])
print p, "concat", q, "=", p.concat(q)

print "Partition enumeration"
for p in Partition.list(6):
        print p

print "Partition branching"
for p in Partition([4, 4, 2, 1]).minus():
        print p

print "Tabloid enumeration"
for p in tabloid_list(Partition([2,2,1])):
        print p

print "Tableau enumeration"
for t in Tableau.list(Partition([3,1])):
        print t

print "Get position in tableau"
t = Tableau.list(Partition([3,2]))[3]
print t
for i in range(5):
        print t.pos(i)

print "Axial distance"
print t.axial_dist(3, 2)

print "Hook formula"
print Tableau.count(Partition([3,1]))

# functions and distributions
print "Zero function"
print Func.zero(3)

print "Product of functions"
print Dist.uniform(3) * Dist.uniform(3)

print "Convolution of functions"
print Dist.uniform(3).convolve(Dist.uniform(3))

print "Uniform distribution"
print Dist.uniform(3)

print "Marginals"
print Dist.uniform(3).marginalize(Partition([3]),
                                  Permutation([0,1,2]),
                                  Permutation([1,0,2]))

print "Marginal matrices"
d = Dist.mallows(Permutation([2, 0, 1]), 0.5)
print d.marginals(Partition([3]))
print d.marginals(Partition([2,1]))
print d.marginals(Partition([1,1,1]))

print "Ordering probabilities"
print Dist.uniform(3).ordered(0, 1)
print Dist.mallows(Permutation([2, 0, 1]), 0.5).orderings()

# mixing and observation models
print "Pairwise mixing model"
print Dist.mix_pair(3, 2, 1, 0.1)

print "k-Subset mixing"
print Dist.mix_subset(4, [0, 1])

print "Insertion mixing"
print Dist.insertion_mix(3)

print "Single track observation"
print Func.singletrack_obs(3, 1, 2, 0.6)

print "Pairwise ranking observation"
print Func.pair_rank_obs(3, 1, 0, 0.4)

print "Mallows model"
print Dist.mallows(Permutation([2, 0, 1]), 0.5)

# independence operations
print "First order join"
j = Dist.join(Dist.uniform(2), Dist.uniform(2))
print j
print tabloid_list(Partition([2,2]))
print j.marginals(Partition([1,1,1,1]))[0]
print j.marginals(Partition([2,2]))

print "First order split"
h,k = j.split(2)
print h
print "--"
print k

# matrices
print "Zero matrix"
print Matrix.zero(3, 5)

print "Identity matrix"
print Matrix.identity(4)

print "Trace"
print Matrix.identity(6).trace()

print "Direct sum"
#print Matrix.direct_sum([Matrix.identity(3), Matrix.zero(4,4)])

# representations and fourier transforms

print "Irrep of an adjacent transposition"
print gz_irrep(Partition([3,2]), Permutation.swap(5, 2, 3))

print "Test homomorphism for some irreps"
m = ((Permutation([0, 2, 3, 1]), Permutation([1, 3, 0, 2])),
     (Permutation([3, 2, 1, 0]), Permutation([0, 3, 2, 1])),
     (Permutation([1, 0, 2, 3]), Permutation([0, 1, 3, 2])))

part = Partition([2, 2])

for (a, b) in m:
        print gz_irrep(part, a.of(b))
        print dot(gz_irrep(part, a), gz_irrep(part, b))

print "Character of an irrep"
ch0 = character(Partition([3,2]))
ch1 = character(Partition([4,1]))
print ch0, ch1
print char_prod(ch0, ch1)
print char_prod(ch0, ch0)
print char_prod(ch1, ch1)

print "Sample distribution"
f = Func.zero(3)
f.set(Permutation.identity(3), 1./3)
f.set(Permutation([1, 0, 2]), 1./6)
f.set(Permutation([0, 2, 1]), 1./3)
f.set(Permutation([1, 2, 0]), 1./6)
print f

print "Restriction"
print f.restrict_to(0)

print "Irreps"
for part in Partition.list(3):
        for i in range(6):
                p = Permutation.from_index(3, i)
                print p, ':', gz_irrep(part, p)

print "Fourier transform"
print f.fourier()

print "Fast Fourier transform"
fft = f.fft()
print fft

print "Inverse Fast Fourier transform"
print fft.inverse_fft()

f = Func.zero(3)
f.set(Permutation.identity(3), 1./2)
f.set(Permutation([1, 0, 2]), 1./2)
print f.fft()
print f.fourier()

print "Reverse fourier transform"
print f.fourier().inverse_fourier()

print "Inverse FFT"
print f.fft().inverse_fft()


     

# catch some errors
try:
        Permutation([2, 3])
except InvalidPermutation:
        print "Caught attempt to create invalid permutation"

try:
        p = Permutation([0, 1, 2])
        p(3)
except IndexOutOfBounds:
        print "Caught index out of bounds"

try:
        Permutation([0, 1]).of(Permutation([0]))
except PermutationMismatch:
        print "Caught permutation mismatch"

try:
        Permutation.from_index(5, 120)
except IndexOutOfBounds:
        print "Caught permutation index out of bounds"

# utilities

print "5! =", fact(5)
